use chrono::{DateTime, Local, Offset};
use derive_more::{Deref, Display, From};
use serde::{Serialize, Serializer, Deserialize, Deserializer};
use std::{
    cmp::Ordering,
    fmt::{Debug, Display, Formatter, Result as FmtResult},
};

#[derive(Copy, Clone, From, Deref, Display)]
#[display("UTC {}", "_0.offset()")]
pub struct FmtOffsetTime(pub DateTime<Local>);

impl Debug for FmtOffsetTime {
    fn fmt(&self, f: &mut Formatter<'_>) -> FmtResult {
        Display::fmt(self, f)
    }
}

impl Serialize for FmtOffsetTime {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer,
    {
        serializer.serialize_str(&self.to_string())
    }
}

impl<'de> Deserialize<'de> for FmtOffsetTime {
    fn deserialize<D>(deserializer: D) -> Result<FmtOffsetTime, D::Error>
    where
        D: Deserializer<'de>,
    {
        // Deserialize the input as a string
        let s = String::deserialize(deserializer)?;

        // Attempt to parse the string into a DateTime<Local>
        match DateTime::parse_from_rfc3339(&s) {
            Ok(dt) => Ok(FmtOffsetTime(dt.with_timezone(&Local))),
            Err(e) => Err(serde::de::Error::custom(format!(
                "Failed to parse DateTime: {}",
                e
            ))),
        }
    }
}

impl Eq for FmtOffsetTime {}
impl PartialEq for FmtOffsetTime {
    fn eq(&self, other: &Self) -> bool {
        self.0.offset().fix() == other.0.offset().fix()
    }
}

impl PartialOrd for FmtOffsetTime {
    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
        Some(
            self.0.offset().fix().local_minus_utc().cmp(&other.0.offset().fix().local_minus_utc()),
        )
    }
}

impl Ord for FmtOffsetTime {
    fn cmp(&self, other: &Self) -> Ordering {
        self.0.offset().fix().local_minus_utc().cmp(&other.0.offset().fix().local_minus_utc())
    }
}
