use super::UNKNOWN;
use crate::modules::{FmtDE, FmtOSArchitecture};
use serde::{Serialize, Deserialize};
use serde_json::{Value, json};
use sysinfo::System;
use thiserror::Error;

#[derive(Debug, Serialize, Deserialize)]
pub struct OSInfo {
    name: String,
    edition: String,
    version: String,
    architecture: FmtOSArchitecture,
    kernel: Option<String>,
    desktop_environment: Option<FmtDE>,
}

#[derive(Debug, Error)]
pub enum OSError {
    #[error("Unsupported pointer width: 16 bit systems are not supported")]
    UnsupportedPointerWidth,
    #[error("Failed to build JSON for OSInfo")]
    JsonError(#[from] serde_json::Error),
}

#[allow(dead_code)]
pub fn get_struct() -> Result<OSInfo, OSError> {
    let architecture = match std::mem::size_of::<usize>() {
        8 => Ok(FmtOSArchitecture::W64_BIT),
        4 => Ok(FmtOSArchitecture::W32_BIT),
        _ => Err(OSError::UnsupportedPointerWidth),
    }?;

    let kernel = if cfg!(target_os = "linux") || cfg!(target_os = "macos") {
        Some(System::kernel_long_version())
    } else {
        None
    };

    let desktop_environment = if cfg!(target_os = "linux") {
        FmtDE::detect()
    } else {
        None
    };

    Ok(OSInfo {
        name: System::name().unwrap_or(UNKNOWN.to_string()),
        edition: System::long_os_version().unwrap_or(UNKNOWN.to_string()),
        version: System::os_version().unwrap_or(UNKNOWN.to_string()),
        architecture,
        kernel,
        desktop_environment,
    })
}

#[allow(dead_code)]
pub fn get_json() -> Result<Value, OSError> {
    Ok(json!(get_struct()?))
}
