use crate::modules::{FmtDateTime, FmtOffsetTime, FmtRelativeTime};
use chrono::{DateTime, Local, Utc, TimeZone};
use serde::{Serialize, Serializer, Deserialize};
use serde_json::{Value, json};
use std::{
    time::Duration as StdDuration,
    fmt::Display,
};
use sysinfo::System;
use thiserror::Error;

fn serialize_datetime<S, Tz>(date: &DateTime<Tz>, serializer: S) -> Result<S::Ok, S::Error>
where
    S: Serializer,
    Tz::Offset: Display,
    Tz: TimeZone,
{
    let formatted = date.format("%+").to_string(); // Format as RFC 3339
    serializer.serialize_str(&formatted)
}

#[derive(Debug, Serialize, Deserialize)]
struct DateInfo {
    #[serde(serialize_with = "serialize_datetime")]
    time_offset: FmtOffsetTime,
    #[serde(serialize_with = "serialize_datetime")]
    local_date_time: FmtDateTime<Local>,
    #[serde(serialize_with = "serialize_datetime")]
    utc_date_time: FmtDateTime<Utc>,
}

#[derive(Debug, Serialize, Deserialize)]
pub struct UptimeInfo {
    boot: DateInfo,
    now: DateInfo,
    relative: FmtRelativeTime,
}

#[derive(Debug, Error)]
pub enum UptimeError {
    #[error("Invalid or out of range timestamp: check seconds < 8_220_000_000_000")]
    InvalidTimestamp,
    #[error("Failed to build JSON for UptimeInfo")]
    JsonError(#[from] serde_json::Error),
}

#[allow(dead_code)]
pub fn get_struct() -> Result<UptimeInfo, UptimeError> {
    let boot_time_utc = DateTime::<Utc>::from_timestamp(System::boot_time() as i64, 0)
        .ok_or(UptimeError::InvalidTimestamp)?;
    let boot_time_local: DateTime<Local> = boot_time_utc.with_timezone(&Local);
    let relative_time =
        StdDuration::from_secs((Local::now() - boot_time_local).num_seconds() as u64);

    let make_info = |utc_dt: DateTime<Utc>| DateInfo {
        time_offset: FmtOffsetTime(Local::now()),
        local_date_time: FmtDateTime(utc_dt.with_timezone(&Local)),
        utc_date_time: FmtDateTime(utc_dt),
    };
    
    Ok(UptimeInfo {
        boot: make_info(boot_time_utc),
        now: make_info(Utc::now()),
        relative: FmtRelativeTime(relative_time),
    })
}

#[allow(dead_code)]
pub fn get_json() -> Result<Value, UptimeError> {
    Ok(json!(get_struct()?))
}
